#|

	This is a file to demonstrate the modeling of the coefficients of the
Chickadee data from the text, Project 2.3.

|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Chickadee Data:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def recruitment
     (/
      (list -478 703 1006 353 620 893 343 578 130 1128 2394 -93 617 996 579
	    492 489 611 734 436 877 492 721)
      10)
     )
(def survival     
     (/
      (list 90 64 85 57 58 64 59 51 69 49 61 54 50 43 75 54 50 50 70 54 48 63
	        55 47)
      100)
     )
(def population
     (/
      (list 2738 1268 1781 2019 1514 1590 1835 1265 1452 850 1643 3281 1557
	        1274 1942 1612 1290 1128 1379 1470 1132 1575 1357 1342)
      10)
     )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Check on relationship between recruitment, survival, and actual pop:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(plot 
 (+ recruitment (* (rest survival) (butlast population)))
 (rest population))

(mprint
 (bind-columns
  recruitment
  (rest survival)
  (+ recruitment (* (rest survival) (butlast population)))
  (rest population))
 )
#|

From this linear relationship, we deduce that the population of the previous
year along with the recruitment and survival numbers for the current year
generate the current year's population: i.e.,

	y(n) = r(n) + s(n)*y(n-1)

|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  histograms (has normal-fit feature):
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def my-plot (histo survival))
(postscript "histo-survival.ps")
(system "ps2gif histo-survival.ps .5")
#|
;; using default 5 bins:
Mean: 0.5425676659334214; Standard Deviation: 8.491017461758291E-2.
|#
(histo recruitment)
(postscript "histo-recruitment.ps")
(system "ps2gif histo-recruitment.ps .5")
#|
;; using default 5 bins:
Mean: 49.22697839212062; Standard Deviation: 47.6657483600559.
with 50 bins:
Mean: 60.33358583351078; Standard Deviation: -22.4203567926868.
|#

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Note that stats (mean and sd) don't correspond to best fitting normal
;;  density (from histo):
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun stats(x) (let ((a (bind-columns x))) (matrix-column-stats a)))
(stats survival)
#|
 ...var .........min ............mean .............max .............std 
 .....1     0.430000         0.591667         0.900000         0.118089     
|#
(stats recruitment)
#|
 ...var .........min ............mean .............max .............std 
 .....1     -47.8000          63.5696          239.400          52.4401     
|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;          Plot for normal and cdf (to illustrate how we use the uniform
;; to generate a sample of random variables)
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(lines on)
(plot #'normal-cdf #'normal-dens -3.5 3.5)
(postscript "normals.ps")
(lines off)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; We use these steps to find the cut-points for the chi-square test: I want
;; the 20% of data points in the middle, 20% in the first ring beyond the
;; middle, etc., so that we end up with 5 different bins.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; add derivative of normal-cdf for root finding:
(add-unary-rule 'normal-cdf #'(lambda (x) (make-unary 'normal-dens x)))

(defun f(x) (- (normal-cdf x) .6))
(roots f .6)
(def p20 ;; % within plus/minus 
     0.25334710313578046)
(defun f(x) (- (normal-cdf x) .7))
(roots f .6)
(def p40 ;; % within plus/minus 
     0.5244005127079024)
(defun f(x) (- (normal-cdf x) .8))
(roots f .6)
(def p60 ;; % within plus/minus 
     0.8416212335729145)
(defun f(x) (- (normal-cdf x) .9))
(roots f .6)
(def p80 ;; % within plus/minus 
     1.2815515655446008)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                chi-square prep
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun chi-counts(p x &key (mean (mean x)) (sd (sd x)))
  "
;; Examples:
 (mapcar
  (lambda(x y) (chi-counts x y))
  (list p20 p40 p60 p80 100)
  (list survival survival survival survival survival)
  )
"
  (let (
	(standard (/ (- x mean) sd))
	)
    (length (which
	     (mapcar 
	      (lambda(x) (and (< x p) (> x (- p))))
	      standard)))
    )
  )

(defun cats(x &key (mean (mean x)) (sd (sd x)))
  "
;; Examples:
 (cats survival)
 (cats recruitment)
"
  (let* ((x 
	  (mapcar
	   (lambda(x y) (chi-counts x y :mean mean :sd sd))
	   (list p20 p40 p60 p80 100)
	   (list x x x x x)
	   )
	  )
	 )
    (combine (first x) (- (rest x) (butlast x)))
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Perform chi-square test:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun chi-square-test(x &key (mean (mean x)) (sd (sd x)))
  "
;; Examples:
 (chi-square-test survival)
 (chi-square-test recruitment)
 (- 1 (chisq-cdf (list 
		  (chi-square-test survival)
		  (chi-square-test recruitment)
	 	  ) 4))

 ;; with 5 bins:
 (chi-square-test survival :mean 0.5425676659334214 :sd 8.491017461758291E-2)
 (chi-square-test recruitment :mean 49.22697839212062 :sd 47.6657483600559)
 (- 1 (chisq-cdf (list 
   (chi-square-test survival :mean 0.5425676659334214 :sd 8.491017461758291E-2)
   (chi-square-test recruitment :mean 49.22697839212062 :sd 47.6657483600559)
	 	  ) 4))

 ;; with 99 bins
 (chi-square-test survival :mean 0.5360970089815541 :sd 0.10057562616432475)
 (chi-square-test recruitment :mean 60.33358583351078 :sd 22.4203567926868)

 ;; we might do a randomization test to see if our chi-square-test results for
 ;; survival and recruitment are extreme: 
 (def ns-check (mapcar (lambda(x) (chi-square-test (normal-rand (length
                        survival))))
		(iseq 1000)))
 (def my-plot (histo ns-check))
 (send my-plot :title (num-to-string (chi-square-test survival)))
 (postscript \"ns-check.ps\")

 (def nr-check (mapcar (lambda(x) (chi-square-test (normal-rand (length
                        recruitment))))
		(iseq 1000)))
 (def my-plot (histo nr-check))
 (send my-plot :title (num-to-string (chi-square-test recruitment)))
 (postscript \"nr-check.ps\")
"
  (let ((exp (/ (length x) 5)))
    (sum
     (/
      (square (- (cats x :mean mean :sd sd) exp))
      exp)
     )
    )
  )

(plot (lambda(x) (chisq-dens x 4)) 0 15)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; quantile and probability plots:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(qqplot (standardize survival))
(send my-plot :add-plot-item (list 'function {x} 'black t))
(postscript "qqsurv.ps" :bulsize 5)
(system "ps2gif qqsurv.ps .5")

(qqplot (standardize recruitment))
(send my-plot :add-plot-item (list 'function {x} 'black t))
(postscript "qqrecruit.ps" :bulsize 5)
(system "ps2gif qqrecruit.ps .5")

(pplot (standardize survival))
(send my-plot :add-plot-item (list 'function {x} 'black t))
(postscript "psurv.ps" :bulsize 5)
(system "ps2gif psurv.ps .5")

(pplot (standardize recruitment))
(send my-plot :add-plot-item (list 'function {x} 'black t))
(postscript "precruit.ps" :bulsize 5)
(system "ps2gif precruit.ps .5")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                68/95/99 rule:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(- (normal-cdf 1) (normal-cdf -1))
;; 0.6826894921370859
(- (normal-cdf 2) (normal-cdf -2))
;; 0.9544997361036416
(- (normal-cdf 3) (normal-cdf -3))
;; 0.9973002039367398
